###############################
###                         ###
###      GRILLMEISTER       ###
###                         ###
### written in 2013 by ZeHa ###
###                         ###
###############################


# Disclaimer:
#
# This game was created as a 2.5 hours game programming challenge.
# I focused solely on finishing a playable and more or less complete game,
# so the code is of course NOT perfect. While it might be useful as an example
# of how to begin writing a game in python/pygame, it is not very useful as a
# general example of "good coding practice".
#
# If you wish, you can contact me at: zeha@drwuro.com


# How to play:
#
# You are a BBQ guy who has to get steaks out of cows that are wandering around
# on your property. Once you got a steak, you have to place it on the grill,
# then go for the next steak. But you have to hurry, otherwise your steak might
# be too well done. If you accomplish this, the game starts over again, just
# with more cows.
# Small hint: You can also go back to the grill to look after your steaks
# if you don't carry a steak at the moment.
#
# Controls:
#
# Arrow Keys = move
# Escape Key = quit
# Space Bar = continue (when level has been won/lost)


import pygame
import random


SCALE = 3
FULLSCREEN = False


manSprite = pygame.image.load('man.png')
fenceSprite = pygame.image.load('fence.png')
grillSprite = pygame.image.load('grill.png')
cowSprite = pygame.image.load('cow.png')
manSteakSprite = pygame.image.load('man-steak.png')

grillSteakSprites = []
grillSteakSprites.append(pygame.image.load('grill-steak.png'))
grillSteakSprites.append(pygame.image.load('grill-steak-2.png'))
grillSteakSprites.append(pygame.image.load('grill-steak-3.png'))
grillSteakSprites.append(pygame.image.load('grill-steak-4.png'))

ohnoSprite = pygame.image.load('ohno.png')
goodjobSprite = pygame.image.load('goodjob.png')

level =  "####################"
level += "#                  #"
level += "#     #            #"
level += "#   ###      #######"
level += "#   #        #     #"
level += "#   #        #     #"
level += "#   #        #     #"
level += "#   #        #     #"
level += "#   #        #     #"
level += "######## #####     #"
level += "#           #      #"
level += "#           #      #"
level += "#                  #"
level += "#           #     G#"
level += "####################"



pygame.display.init()

flags = pygame.DOUBLEBUF | pygame.HWSURFACE
if FULLSCREEN:
    flags |= pygame.FULLSCREEN

realScreen = pygame.display.set_mode((320 * SCALE, 240 * SCALE), flags, 32)
pygame.display.set_caption('Grillmeister')
screen = pygame.Surface((320, 240))

numCows = 3
grillTime = 60
grillPhases = len(grillSteakSprites)

cowX = []
cowY = []

hasSteak = False
steakOnGrill = 0


def init():
    global manX, manY, hasSteak, steakOnGrill, cowX, cowY
    manX = 17
    manY = 13

    hasSteak = False
    steakOnGrill = 0

    cowX = []
    cowY = []

    for c in range(numCows):
        cx = 0
        cy = 0

        while level[cy * 20 + cx] != ' ' or (cx in cowX and cy in cowY) or (cowX == manX and cowY == manY):
            cx = int(random.random() * 20)
            cy = int(random.random() * 15)

        cowX.append(cx)
        cowY.append(cy)

def the_game():
    global manX, manY, hasSteak, steakOnGrill

    # render --------------------------------------
    screen.fill((0, 168, 0))

    for y in range(15):
        for x in range(20):
            if level[y * 20 + x] == '#':
                screen.blit(fenceSprite, (x * 16, y * 16))
            elif level[y * 20 + x] == 'G':
                if steakOnGrill:
                    i = grillPhases - int((float(grillPhases) / grillTime) * steakOnGrill) -1
                    screen.blit(grillSteakSprites[i], (x * 16, y * 16))
                else:
                    screen.blit(grillSprite, (x * 16, y * 16))

    if hasSteak:
        screen.blit(manSteakSprite, (manX * 16, manY * 16))
    else:
        screen.blit(manSprite, (manX * 16, manY * 16))

    for c in range(numCows):
        screen.blit(cowSprite, (cowX[c] * 16, cowY[c] * 16))

    # man -----------------------------------------
    update = False

    for event in pygame.event.get():
        if event.type == pygame.KEYUP:
            if event.key == pygame.K_ESCAPE:
                return 'quit'

        elif event.type == pygame.KEYDOWN:
            newX = manX
            newY = manY
            update = True

            if event.key == pygame.K_LEFT:
                newX -= 1
            if event.key == pygame.K_RIGHT:
                newX += 1
            if event.key == pygame.K_UP:
                newY -= 1
            if event.key == pygame.K_DOWN:
                newY += 1

            if level[newY * 20 + newX] != '#':
                manX = newX
                manY = newY

            if level[newY * 20 + newX] == 'G':
                steakOnGrill = grillTime
                if hasSteak:
                    hasSteak = False

    # cows ----------------------------------------
    if update:
        for c in range(numCows):
            if cowX[c] == manX and cowY[c] == manY:
                if not hasSteak:
                    cowX[c] = -1
                    cowY[c] = -1
                    hasSteak = True

            if cowX[c] > 0:
                while True:
                    newX = cowX[c]
                    newY = cowY[c]

                    direction = int(random.random() * 4)

                    if direction == 0:      # left
                        newX -= 1
                    elif direction == 1:    # right
                        newX += 1
                    elif direction == 2:    # up
                        newY -= 1
                    elif direction == 3:    # down
                        newY += 1

                    if level[newY * 20 + newX] == ' ':
                        cowX[c] = newX
                        cowY[c] = newY
                        break

            if cowX[c] == manX and cowY[c] == manY:
                if not hasSteak:
                    cowX[c] = -1
                    cowY[c] = -1
                    hasSteak = True

    # grill ---------------------------------------
    if update:
        if not hasSteak:
            remainingCows = numCows
            for c in range(numCows):
                if cowX[c] == -1:
                    remainingCows -=1

            if remainingCows == 0:
                return 'won'                    

        if steakOnGrill:
            if steakOnGrill == 1:
                return 'lost'
            else:
                steakOnGrill -= 1

    return 'play'

def game_won():
    screen.fill((0, 168, 0))

    for y in range(15):
        for x in range(20):
            if level[y * 20 + x] == 'G':
                screen.blit(grillSteakSprites[0], (x * 16, y * 16))
                screen.blit(goodjobSprite, ((x - 3) * 16, y * 16))

    screen.blit(manSprite, ((manX -1) * 16, manY * 16))

    for event in pygame.event.get():
        if event.type == pygame.KEYUP:
            if event.key == pygame.K_SPACE:
                global numCows
                numCows += 2
                return 'new'

    return 'won'

def game_over():
    screen.fill((0, 168, 0))

    for y in range(15):
        for x in range(20):
            if level[y * 20 + x] == 'G':
                screen.blit(grillSteakSprites[grillPhases -1], (x * 16, y * 16))
                screen.blit(ohnoSprite, ((x - 1) * 16, y * 16))

    for event in pygame.event.get():
        if event.type == pygame.KEYUP:
            if event.key == pygame.K_SPACE:
                return 'new'

    return 'lost'

def game_loop():
    state = 'new'

    while True:
        if state == 'new':
            init()
            state = 'play'
        elif state == 'play':
            state = the_game()
        elif state == 'won':
            state = game_won()
        elif state == 'lost':
            state = game_over()
        elif state == 'quit':
            break

        pygame.transform.scale(screen, (320 * SCALE, 240 * SCALE), realScreen)
        pygame.display.flip()

    pygame.quit()

game_loop()

